<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://resoc.io
 * @since      1.0.0
 *
 * @package    Resoc
 * @subpackage Resoc/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Resoc
 * @subpackage Resoc/admin
 * @author     Philippe Bernard <philippe@resoc.io>
 */
class Resoc_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $resoc    The ID of this plugin.
	 */
	private $resoc;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $resoc       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $resoc, $version ) {

		$this->resoc = $resoc;
		$this->version = $version;

    add_action( 'save_post',
      array( $this, 'save_social_image' ) );

    add_action( 'admin_notices', array( $this, 'finish_setup_notice' ));

    add_filter('plugin_action_links', array( $this, 'plugin_action_links' ), 10, 2);
  }

  public function plugin_action_links( $links, $file ) {
    static $this_plugin;

    if ( ! $this_plugin ) {
        $this_plugin = resoc_plugin_base_name();
    }

    if ( $file == 'resoc/resoc.php' ) {
      $settings_link = '<a href="' . admin_url( '/themes.php?page=resoc_social_images_appearance_menu' ) . '">' . 'Settings' . '</a>';
      array_unshift( $links, $settings_link );
    }

    return $links;
  }

  public function save_social_image( $post_ID ) {
    Resoc_Utils::log( "Generate social images for post " . $post_ID );

    $post = get_post( $post_ID );
    if ( $post->post_type == 'revision' ) {
      Resoc_Utils::log( "Post " . $post_ID . " is a revision, skip" );
      return;
    }

    $feature_image_url = Resoc_Utils::get_post_featured_image_url( $post_ID );
    if ( !$feature_image_url ) {
      Resoc_Utils::log( "Cannot get featured image for post " . $post_ID );
      return;
    }

    $featured_image = Resoc_Utils::image_url_to_base64( $feature_image_url );
    if ( !$featured_image ) {
      Resoc_Utils::log( "Cannot turn featured image URL (" . $feature_image_url . ") to Base64" );
      return;
    }

    if ( !Resoc_Utils::plugin_configured() ) {
      Resoc_Utils::log( "Plugin is not configured" );
      return;
    }

    $plugin_options = get_option( Resoc_Settings::OPTIONS );
    $request = array(
      'template' => $plugin_options[ Resoc_Settings::TEMPLATE ],
      'parameters' => array(
        'title' => $post->post_title,
        'backgroundColor' => $plugin_options[ Resoc_Settings::BACKGROUND_COLOR ],
        'textColor' => $plugin_options[ Resoc_Settings::TEXT_COLOR ],
        'mainImageUrl' => $featured_image,
        'logoUrl' => Resoc_Utils::get_logo_as_base64(),
        'brandName' => $plugin_options[ Resoc_Settings::BRAND_NAME ],
        'textDirection' => isset( $plugin_options[ Resoc_Settings::TEXT_DIRECTION ] )
          ? $plugin_options[ Resoc_Settings::TEXT_DIRECTION ]
          : 'ltr'
      )
    );

    $current_request_hash = Resoc_Utils::hash_request( $request );
    $previous_request_hash = get_post_meta( $post_ID, Resoc::POST_META_LATEST_REQUEST_HASH, true );
    if ( $previous_request_hash && $previous_request_hash == $current_request_hash ) {
      Resoc_Utils::log( "No change since the last time the image was generated, do nothing" );
      return;
    }

    $social_image = Resoc_Utils::create_social_image( $request );
    if ( !$social_image ) {
      Resoc_Utils::log( "Cannot generate social image for post " . $post_ID );
      return;
    }

    // Get existing OpenGraph image to update it as an attachement
    $existing_og_image_id = get_post_meta( $post_ID, Resoc::POST_META_OG_IMAGE_ID, true );

    $social_image_id = Resoc_Utils::add_image_to_media_library( $social_image, $post_ID, $existing_og_image_id );
    if ( !$social_image_id ) {
      Resoc_Utils::log( "Cannot store/attach social image for post " . $post_ID );
      return;
    }

    update_post_meta( $post_ID, Resoc::POST_META_OG_IMAGE_ID, $social_image_id );

    update_post_meta( $post_ID, Resoc::POST_META_LATEST_REQUEST_HASH, $current_request_hash );

    Resoc_Utils::log( "OpenGraph image " . $social_image_id . " attached to post " . $post_ID );
  }

  public function finish_setup_notice() {
    if ( Resoc_Utils::plugin_configured() ) {
      return;
    }

    $settings_page_url = admin_url( '/themes.php?page=resoc_social_images_appearance_menu' );
    if ( Resoc_Utils::current_url() == $settings_page_url ) {
      return;
    }

    ?>
    <div class="notice notice-warning is-dismissible">
      <p>
        <strong>Resoc Social Images</strong> is almost ready!
        <a href="<?php echo $settings_page_url ?>">Choose your template</a>
        to complete the setup.
      </p>
    </div>
    <?php
  }

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Resoc_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Resoc_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->resoc, plugin_dir_url( __FILE__ ) . 'css/resoc-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Resoc_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Resoc_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->resoc, plugin_dir_url( __FILE__ ) . 'js/resoc-admin.js', array( 'jquery' ), $this->version, false );

	}

}
