<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://resoc.io
 * @since      1.0.0
 *
 * @package    Resoc
 * @subpackage Resoc/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Resoc
 * @subpackage Resoc/admin
 * @author     Philippe Bernard <philippe@resoc.io>
 */
class Resoc_Settings {

  const OPTIONS = 'resoc_options';

  const TEMPLATE         = 'template';
  const BACKGROUND_COLOR = 'background-color';
  const TEXT_COLOR       = 'text-color';
  const BRAND_NAME       = 'brand-name';
  const LOGO             = 'logo';
  const TEXT_DIRECTION   = 'textDirection';

  /**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $resoc    The ID of this plugin.
	 */
	private $resoc;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $resoc       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $resoc, $version ) {
		$this->resoc = $resoc;
		$this->version = $version;

    add_action( 'admin_menu',
      array( $this, 'add_appearance_menu' ) );

    add_action( 'admin_init',
      array( $this, 'init_settings' ) );

    add_action( 'admin_enqueue_scripts',
      array( $this, 'enqueue_field_resources' ) );
  }

  public function add_appearance_menu() {
    if ( ! current_user_can( 'manage_options' ) ) {
      Resoc_Utils::log( "User does not have manage_options capability, skip settings" );
      return;
    }

    add_theme_page( 'Social Images', 'Social Images', 'manage_options',
      'resoc_social_images_appearance_menu',
      array( $this, 'create_social_images_appearance_page' ) );
  }

  public function create_social_images_appearance_page() {
    if ( ! current_user_can( 'manage_options' ) ) {
      Resoc_Utils::log( "User does not have manage_options capability, skip settings" );
      return;
    }

    if ( Resoc_Compatibility::conflicting_plugin() ) {
      include_once( plugin_dir_path(__FILE__) . 'partials' . DIRECTORY_SEPARATOR . 'appearance-conflict-fallback.php' );
      return;
    }

    include_once( plugin_dir_path(__FILE__) . 'partials' . DIRECTORY_SEPARATOR . 'appearance.php' );
  }

  public function init_settings() {
    register_setting( 'resoc', Resoc_Settings::OPTIONS );

    add_settings_section(
      'resoc_section',
      'Template', array( $this, 'resoc_section_callback' ),
      'resoc'
    );

    add_settings_field(
      Resoc_Settings::TEMPLATE,
      'Template',
      array( $this, 'field_template_callback' ),
      'resoc',
      'resoc_section',
      array(
        'label_for'         => Resoc_Settings::TEMPLATE,
        'class'             => 'resoc-row'
      )
    );

    add_settings_field(
      Resoc_Settings::BACKGROUND_COLOR,
      'Background color',
      array( $this, 'field_background_color_callback' ),
      'resoc',
      'resoc_section',
      array(
        'label_for'         => Resoc_Settings::BACKGROUND_COLOR,
        'class'             => 'resoc-row'
      )
    );

    add_settings_field(
      Resoc_Settings::TEXT_COLOR,
      'Text color',
      array( $this, 'field_text_color_callback' ),
      'resoc',
      'resoc_section',
      array(
        'label_for'         => Resoc_Settings::TEXT_COLOR,
        'class'             => 'resoc-row'
      )
    );

    add_settings_field(
      Resoc_Settings::LOGO,
      'Logo',
      array( $this, 'field_logo_callback' ),
      'resoc',
      'resoc_section',
      array(
        'label_for'         => Resoc_Settings::LOGO,
        'class'             => 'resoc-row'
      )
    );

    add_settings_field(
      Resoc_Settings::BRAND_NAME,
      'Brand or company name',
      array( $this, 'field_brand_name_callback' ),
      'resoc',
      'resoc_section',
      array(
        'label_for'         => Resoc_Settings::BRAND_NAME,
        'class'             => 'resoc-row'
      )
    );

    add_settings_field(
      Resoc_Settings::TEXT_DIRECTION,
      'Text direction',
      array( $this, 'field_text_direction_callback' ),
      'resoc',
      'resoc_section',
      array(
        'label_for'         => Resoc_Settings::TEXT_DIRECTION,
        'class'             => 'resoc-row'
      )
    );
  }

  function resoc_section_callback( $args ) {
    ?>
      <div id="<?php echo esc_attr( $args['id'] ); ?>">
        <p>
          The template is used to turn you post featured images into branded images for social networks:
        </p>

        <?php
          include_once( plugin_dir_path(__FILE__) . 'partials' . DIRECTORY_SEPARATOR . 'before-after.php' );
        ?>

        <p>
          Customize the template so it fits <em>your</em> website and brand.
        </p>
      </div>
    <?php
  }

  function field_template_callback( $args ) {
    $options = get_option( 'resoc_options' );

    ?>
      <div class="resoc-template-list">
    <?php

    foreach( [ 'basic01', 'basic02', 'basic03' ] as $template ) {
      ?>
        <div class="resoc-template">
          <input
            type="radio"
            id="<?php echo esc_attr( $args['label_for'] . '-' . $template ); ?>"
            name="resoc_options[<?php echo esc_attr( $args['label_for'] ); ?>]"
            value="<?php echo esc_attr( $template ) ?>"
            <?php echo isset( $options[ $args['label_for'] ] ) ? ( checked( $options[ $args['label_for'] ], $template, false ) ) : ( '' ); ?>
          />
          <label for="<?php echo esc_attr( $args['label_for'] . '-' . $template ); ?>">
            <?php echo esc_html( ucfirst( $template ) ) ?>
            <br />
            <img
              class="resoc-template-illustration"
              src="<?php echo esc_attr( plugin_dir_url( __FILE__ ) . 'img/' . $template . '.jpg' ) ?>"
            />
          </label>
        </div>
      <?php
    }

    ?>
      </div>
    <?php
  }

  function field_background_color_callback( $args ) {
    $options = get_option( 'resoc_options' );

    ?>
      <input
        type="text"
        id="<?php echo esc_attr( $args['label_for'] ); ?>"
        name="resoc_options[<?php echo esc_attr( $args['label_for'] ); ?>]"
        value="<?php echo esc_attr( isset( $options[ $args['label_for'] ] ) ? $options[ $args['label_for'] ] : '#ffb703' ) ?>"
        class="resoc-color-picker"
        data-default-color="#ffb703"
      />
    <?php
  }

  function field_text_color_callback( $args ) {
    $options = get_option( 'resoc_options' );

    ?>
      <input
        type="text"
        id="<?php echo esc_attr( $args['label_for'] ); ?>"
        name="resoc_options[<?php echo esc_attr( $args['label_for'] ); ?>]"
        value="<?php echo esc_attr( isset( $options[ $args['label_for'] ] ) ? $options[ $args['label_for'] ] : '#ffffff' ) ?>"
        class="resoc-color-picker"
        data-default-color="#ffffff"
      />
    <?php
  }

  public function enqueue_field_resources() {
    // Color picker
    wp_enqueue_style( 'wp-color-picker' );
    wp_enqueue_script( 'wp-color-picker', admin_url( 'js/color-picker.min.js' ), array( 'iris' ) );
    wp_enqueue_script( 'resoc-settings', plugin_dir_url( __FILE__ ) . 'js/resoc-settings.js', array( 'jquery', 'wp-color-picker' ) );

    // Media picker
    wp_enqueue_style( 'wp-components' );
    wp_enqueue_media();
  }

  function field_logo_callback( $args ) {
    $options = get_option( 'resoc_options' );

    $image_url = NULL;
    if ( isset( $options[ $args['label_for'] ] ) ) {
      $image_url = wp_get_attachment_url( $options[ $args['label_for'] ] );
    }
    $image_id = $options[ $args['label_for'] ];

    ?>
      <div class="resoc-media-field">
        <input
          type="hidden"
          id="<?php echo esc_attr( $args['label_for'] ); ?>"
          name="resoc_options[<?php echo esc_attr( $args['label_for'] ); ?>]"
          value="<?php echo esc_attr( isset( $options[ $args['label_for'] ] ) ? $options[ $args['label_for'] ] : '' ) ?>"
          class="media-id"
        />
        <img
          class="media-thumbnail <?php $image_url ? '' : 'hidden-media-thumbnail' ?>"
          src="<?php echo esc_attr( $image_url ) ?>"
        />
        <button type="button" class="components-button is-secondary media-picker-button">
          <?php echo esc_html( $image_url ? 'Replace logo' : 'Set logo' ) ?>
        </button>
      </div>
    <?php
  }

  function field_brand_name_callback( $args ) {
    $options = get_option( 'resoc_options' );

    ?>
      <input
        type="text"
        id="<?php echo esc_attr( $args['label_for'] ); ?>"
        name="resoc_options[<?php echo esc_attr( $args['label_for'] ); ?>]"
        value="<?php echo esc_attr( isset( $options[ $args['label_for'] ] ) ? $options[ $args['label_for'] ] : '' ) ?>"
      />
    <?php
  }

  function field_text_direction_callback( $args ) {
    $options = get_option( 'resoc_options' );
    $text_direction = isset( $options[ $args['label_for'] ] )
      ? $options[ $args['label_for'] ]
      : 'ltr';

    ?>
      <fieldset class="resoc-text-direction-list">
    <?php

    foreach( [
      array( 'slug' => 'ltr', 'name' => 'Left-to-Right (English, Spanish...)' ),
      array( 'slug' => 'rtl', 'name' => 'Right-to-Left (Arabic, Hebrew...)' )
    ] as $direction ) {
      ?>
        <div class="resoc-text-direction">
          <input
            type="radio"
            id="<?php echo esc_attr( $args['label_for'] . '-' . $direction[ 'slug' ] ); ?>"
            name="resoc_options[<?php echo esc_attr( $args['label_for'] ); ?>]"
            value="<?php echo esc_attr( $direction[ 'slug' ] ) ?>"
            <?php echo checked( $text_direction, $direction[ 'slug' ], false ) ?>
          />
          <label for="<?php echo esc_attr( $args['label_for'] . '-' . $direction[ 'slug' ] ); ?>">
            <?php echo esc_html( ucfirst( $direction[ 'name' ] ) ) ?>
          </label>
        </div>
      <?php
    }

    ?>
      </fieldset>
    <?php
  }

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Resoc_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Resoc_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->resoc, plugin_dir_url( __FILE__ ) . 'css/resoc-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Resoc_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Resoc_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->resoc, plugin_dir_url( __FILE__ ) . 'js/resoc-admin.js', array( 'jquery' ), $this->version, false );

	}

}
